;
; 
; 10-01-2008
;
; The program sorts 5 integers.
; It uses PR15 as output and RP14 as input.
; The last processot uses RP2 as output for communication with the PC.
;
;******************************************************************************

               .equ __24FJ16GA002, 1
               .include "p24FJ16GA002.inc"

;..............................................................................
; Declarations for conditional assembly
;..............................................................................

.equ First, 1
.equ Middle, 2
.equ Last, 3


.set Processor, First
;..............................................................................
; Constants:
;..............................................................................

.equ MAXINT, 0x00FF
;..............................................................................
;Global Declarations:
;..............................................................................


        .global __reset          ;The label for the first line of code. 

        .global __T1Interrupt    ;Declare Timer 1 ISR name global

;..............................................................................
;Uninitialized variables in Near data memory (Lower 8Kb of RAM)
;..............................................................................

          .section .nbss, bss, near
send_data:     .space 2               ;1 word of space for variable "send_data".




;..............................................................................
; Macro section
;..............................................................................

.macro wait_long				;wait for 3 msec @ 2MHZ (> 1.2 msec = 1 byte @ 9600)

		PUSH	W8

		MOV	#1000,W8
2:
		NOP
		DEC	W8,W8
		BRA	NZ,2b

		POP	W8
.endm

.macro wait_sort

		PUSH	W8

		MOV	#100,W8
2:
		NOP
		DEC	W8,W8
		BRA	NZ,2b

		POP	W8
.endm

;=======================




;..............................................................................
;Code Section in Program Memory
;..............................................................................

.text                             ;Start of Code section
__reset:
        MOV #__SP_init, W15       ;Initalize the Stack Pointer
        MOV #0x17F8, W0     ;Initialize the Stack Pointer Limit Register
        MOV W0, SPLIM
        NOP                       ;Add NOP to follow SPLIM initialization
        
        CALL _wreg_init           ;Call _wreg_init subroutine
                                  ;Optionally use RCALL instead of CALL




        ;<<insert more user code here>>


;
; initialize
;
		MOV #0xFFFF,W1
		MOV W1,AD1PCFG			; turn off the Analog nature of Port A pins

		MOV	#0xFFFF, W1			; set PORTA as input.
		MOV	W1,TRISA


		CALL	Clear_all_RPx_outputs


		MOV	#0x1F0E,W1
		MOV	W1,RPINR18			; RxD --> RP14

		MOV	#0x0300,W1
		MOV	W1,RPOR7			; TxD --> RP15

		MOV	#0x0000,W1
		MOV	W1,U1BRG			; BRG <-- MAX
		MOV	#0x0008,W1			; 0000 0000 0000 1000
		MOV	W1,U1MODE			; setup UART: UARTEN=0, RXINV=0, BRGH=1, 8-bit, no parity, 1 stop.
		MOV	#0x8008,W1
		MOV	W1,U1MODE			; Enable UART
		MOV	#0x04C0,W1
		MOV	W1,U1STA			; setup UART


		MOV	#0x0008,W1
		MOV	W1,U1MODE
		MOV	#0x8008,W1
		MOV	W1,U1MODE
		MOV	#0x04C0,W1
		MOV	W1,U1STA			; setup UART


; main code
.if (Processor == First)

; W2 : send_data
; W3 : local_data
; W4 : read_data
; W5 ; loop counter
; W6,W7,W8,W9,W10 = data
; W11 : index of A[]

; 
; PHASE 1
;

		MOV	#0x0036,W6
		MOV	#0x0032,W7
		MOV	#0x0039,W8
		MOV	#0x0031,W9
		MOV	#0x0037,W10

		MOV	#0x000C,W11
		NOP
		MOV	[W11++],W2			;send_data <-- A[1]

		MOV	#0x0005,W5		;For i = 1 to 5                    /* N=5 */
loopF1:
		NOP
		MOV	W2,U1TXREG		;SEND(Right)  <--  send_data
		wait_sort
		MOV	U1RXREG,W4		;read_data <-- RECEIVE(Left)


		MOV	[W11++],W2			;send_data <-- A[i]
		NOP
		NOP							
		NOP
		NOP
    
		DEC W5,W5
		BRA	NZ,loopF1		;Next	i


		MOV	#0x0004,W5		;For i = 1 to 4                    /* N=5 ==> (N-1) = 4*/
loopF2:
		MOV	#MAXINT,W2			;send_data <-- MAX
		MOV	W2,U1TXREG		;SEND(Right)  <--  send_data
		wait_sort
		NOP

		NOP
		NOP
		NOP							
		NOP
		NOP
    
		DEC W5,W5
		BRA	NZ,loopF2		;Next	i


; 
; PHASE 2
;
		NOP
		MOV	#0x0005,W5		;For i = 1 to 5                    /* N=5 */
loopF3:
		MOV	#MAXINT,W2			;	send_data  <--  MAX
		MOV	W2,U1TXREG		;	SEND(Right)  <--  send_data
		wait_sort
		NOP
		NOP
		DEC W5,W5
		BRA	NZ,loopF3		;Next	i


.elseif (Processor == Middle)

; W2 : send_data
; W3 : local_data
; W4 : read_data
; W5 ; loop counter

; 
; PHASE 1
;

		NOP
		NOP
		NOP
		NOP
		NOP
		NOP

		MOV	#MAXINT,W2		;send_data <-- M
		MOV	#MAXINT,W3		;local_data <-- M

		MOV	#0x0009,W5		;For i = 1 to 9                    /* N=5 ==> 2N-1 =9 */
loopM:
		NOP
		MOV	W2,U1TXREG		;SEND(Right)  <--  send_data
		wait_sort
		MOV	U1RXREG,W4		;read_data <-- RECEIVE(Left)


		CP	W4,W3			;if  read_data  >  local_data  then 
		BRA	LE,labA
		MOV	W4,W2			;	send_data  <--  read_data
							;	local_data <--  local_data
		BRA	labB
labA:						;else
		MOV	W3,W2			;	send_data  <--  local_data
		MOV W4,W3			;	local_data <--  read_data
labB:						;endif
    
		DEC W5,W5
		BRA	NZ,loopM		;Next	i


; 
; PHASE 2
;
		NOP
		MOV	#0x0005,W5		;For i = 1 to 5                    /* N=5 */
loopM2:
		MOV	W3,W2			;	send_data  <--  local_data
		MOV	W2,U1TXREG		;	SEND(Right)  <--  send_data
		wait_sort
		MOV	U1RXREG,W4		;	read_data <-- RECEIVE(Left)
		MOV W4,W3			;	local_data <--  read_data
		DEC W5,W5
		BRA	NZ,loopM2		;Next	i


.elseif (Processor == Last)

; W2 : send_data
; W3 : local_data
; W4 : read_data
; W5 ; loop counter
; W6,W7,W8,W9,W10 = data
; W11 : index of A[]

; 
; PHASE 1
;


		NOP
		NOP
		NOP
		NOP
		NOP
		NOP
		NOP
		NOP


		MOV	#0x0009,W1		;For i = 1 to 9                    /* N=5 ==> 2N-1 =9 */
loopL1:
		NOP
		NOP
		wait_sort
	
		MOV	U1RXREG,W2		;read_data <-- RECEIVE(Left)


		NOP
		NOP
		NOP
		NOP
		NOP
    
		DEC W1,W1
		BRA	NZ,loopL1		;Next	i


; 
; PHASE 2
;

		MOV	#0x000C,W12
		MOV	#0x0005,W1		;For i = 1 to 5                    /* N=5 */
loopL2:
		NOP
		NOP
		wait_sort
		MOV	U1RXREG,W2		;	read_data <-- RECEIVE(Left)

		MOV	W2,[W12++]		;   B[i] <-- read_data

		DEC W1,W1
		BRA	NZ,loopL2		;Next	i

;
; Transmit ot PC phase
;


		MOV	#0x0008,W1			; 0000 0000 0000 1000
		MOV	W1,U1MODE			; setup UART: UARTEN=0, RXINV=0, BRGH=1, 8-bit, no parity, 1 stop.


		MOV	#0x0003,W1
		MOV	W1,RPOR1			; TxD --> RP2

		MOV	#0x0019,W1
		MOV	W1,U1BRG			; BRG <-- 19(hex) = 25(dec) (baud rate = 9600 @ 2MHZ with BRGH=1)
		MOV	#0x0008,W1			; 0000 0000 0000 1000
		MOV	W1,U1MODE			; setup UART: UARTEN=0, RXINV=0, BRGH=1, 8-bit, no parity, 1 stop.

		MOV	#0x8008,W1
		MOV	W1,U1MODE			; Enable UART
		MOV	#0x04C0,W1
		MOV	W1,U1STA			; setup UART



; send data

MOV #0x003E,W1		;'>'
MOV	W1,U1TXREG
wait_long

MOV	W6,U1TXREG
wait_long
MOV #0x002C,W1		;','
MOV	W1,U1TXREG
wait_long

MOV	W7,U1TXREG
wait_long
MOV #0x002C,W1		;','
MOV	W1,U1TXREG
wait_long

MOV	W8,U1TXREG
wait_long
MOV #0x002C,W1		;','
MOV	W1,U1TXREG
wait_long

MOV	W9,U1TXREG
wait_long
MOV #0x002C,W1		;','
MOV	W1,U1TXREG
wait_long

MOV	W10,U1TXREG
wait_long
MOV #0x002E,W1		;'.'
MOV	W1,U1TXREG
wait_long

MOV #0x000D,W1		;CR
MOV	W1,U1TXREG
wait_long
MOV #0x000A,W1		;LF
MOV	W1,U1TXREG
wait_long



.endif



;---------------------------------------------------------------

done:
        BRA     done              ;Place holder for last line of executed code



;..............................................................................
;Subroutine: Clear RP2-RP15 pins Output (==> set all RPORx to 0000)
;..............................................................................
Clear_all_RPx_outputs:

		PUSH	W1

		MOV		#0x0000,W1
		MOV		W1,RPOR1		; clear RP3, RP2
		MOV		W1,RPOR2		; clear RP5, RP4
		MOV		W1,RPOR3		; clear RP7, RP6
		MOV		W1,RPOR4		; clear RP9, RP8
		MOV		W1,RPOR5		; clear RP11, RP10
		MOV		W1,RPOR6		; clear RP13, RP12
		MOV		W1,RPOR7		; clear RP15, RP14

		POP		W1
		RETURN
;..............................................................................
;Subroutine: Initialization of W registers to 0x0000
;..............................................................................

_wreg_init:
        CLR W0
        MOV W0, W14
        REPEAT #12
        MOV W0, [++W14]
        CLR W14
        RETURN



;..............................................................................
;Timer 1 Interrupt Service Routine
;Example context save/restore in the ISR performed using PUSH.D/POP.D
;instruction. The instruction pushes two words W4 and W5 on to the stack on
;entry into ISR and pops the two words back into W4 and W5 on exit from the ISR
;..............................................................................

__T1Interrupt:
        PUSH.D W4                  ;Save context using double-word PUSH

        ;<<insert more user code here>>

        BCLR IFS0, #T1IF           ;Clear the Timer1 Interrupt flag Status
                                   ;bit.
        POP.D W4                   ;Retrieve context POP-ping from Stack
        RETFIE                     ;Return from Interrupt Service routine



;--------End of All Code Sections ---------------------------------------------

.end                               ;End of program code in this file



